Class: RobotLab::RobotSpec
- Inherits:
-
Data
- Object
- Data
- RobotLab::RobotSpec
- Defined in:
- lib/robot_lab/ractor_job.rb
Overview
Carries everything needed to reconstruct a Robot inside a Ractor. All fields must be frozen strings, symbols, or hashes.
Instance Attribute Summary collapse
-
#config_hash ⇒ Object
readonly
Returns the value of attribute config_hash.
-
#name ⇒ Object
readonly
Returns the value of attribute name.
-
#system_prompt ⇒ Object
readonly
Returns the value of attribute system_prompt.
-
#template ⇒ Object
readonly
Returns the value of attribute template.
Instance Attribute Details
#config_hash ⇒ Object (readonly)
Returns the value of attribute config_hash
36 37 38 |
# File 'lib/robot_lab/ractor_job.rb', line 36 def config_hash @config_hash end |
#name ⇒ Object (readonly)
Returns the value of attribute name
36 37 38 |
# File 'lib/robot_lab/ractor_job.rb', line 36 def name @name end |
#system_prompt ⇒ Object (readonly)
Returns the value of attribute system_prompt
36 37 38 |
# File 'lib/robot_lab/ractor_job.rb', line 36 def system_prompt @system_prompt end |
#template ⇒ Object (readonly)
Returns the value of attribute template
36 37 38 |
# File 'lib/robot_lab/ractor_job.rb', line 36 def template @template end |